/**
 * @file
 * @author Remi Bloch
 * @author Thomas Schwery
 */

#include "Activity.h"

/**
 * Constructor for an activity
 *
 * @param name		The human readable name of the activity
 * @param category      The human-readable category of the activity
 */
Activity::Activity(QString name, QString category) {
	this->name = name;
        this->category = category;
}

Activity::Activity(const Activity & that) {
	this->name = that.name;
        this->category = that.category;
}

Activity::~Activity() {
}

/**
 * Returns the name of the activity
 *
 * @return 		The name
 */
QString Activity::getName() const {
	return this->name;
}

/**
 * Returns the name of the category
 *
 * @return 		The category
 */
QString Activity::getCategory() const {
        return this->category;
}

/**
 * Returns the hash of the activity
 *
 * @return		The hash
 */
QString Activity::getHash() const {
        return QCryptographicHash::hash(QString(name+category).toUtf8(),QCryptographicHash::Sha1);
}

/**
 * Returns an xml-format description of the hashed activity in the form
 *
 * @code
 * <activity hash='hash' />
 * @endcode
 *
 * @return		An xml description.
 */
QString Activity::toHashedXMLEntry() const {
        return "<activity hash='" + getHash() +"' />";
}

/**
 * Returns an xml-format description of the activity in the form
 *
 * @code
 * <activity id='name' category='category' />
 * @endcode
 *
 * @return		An xml description.
 */
QString Activity::toXMLEntry() const {
        return "<activity name='" + name +"' category='" + category + "' />";
}

QTextStream &operator<<(QTextStream &out, const Activity &activity) {
        out << activity.getHash() << "(" << activity.getName() << " - " << activity.getCategory() << ")";
	return out;
}
/**
 * Override to be able to compare two activities.
 *
 * 2 activities are equal if their identifier and their description are
 * equal.
 *
 * @return		true if the two activities are equals, false otherwise.
 */
bool Activity::operator==(const Activity &toCompare) const {
        return toCompare.getHash() == this->getHash();
}

bool Activity::operator!=(const Activity &toCompare) const {
        return toCompare.getHash() != this->getHash();
}

Activity& Activity::operator=(const Activity & that) {
        if (this != &that) {
                this->name = that.name;
                this->category = that.category;
	}
	return *this;
}

bool Activity::operator<(const Activity &toCompare) const {
	return this->name < toCompare.name;
}


uint qHash(const Activity &activity){
     return qHash(activity.getName() + activity.getCategory());
}
