/*
 * RequestsManager.cpp
 *
 *  Created on: Oct 19, 2010
 *      Author: maemo
 */

#include "RequestsManager.h"

RequestsManager * RequestsManager::_instance = NULL;

RequestsManager::RequestsManager() {
	requests = new QHash<QString,Request>();
}

void RequestsManager::setChannelSender(ChannelSender * sender) {
	this->sender = sender;
}

/**
 * Returns an instance of the manager.
 *
 * @return		An instance
 */
RequestsManager * RequestsManager::getInstance() {
	if (!_instance) {
		_instance = new RequestsManager();
	}
	return _instance;
}

/**
 * Register a new request
 *
 * @param request		The request to add
 *
 * @return				true
 */
bool RequestsManager::addRequest(const Request &request) {
	QString requestIdentifier = request.getIdentifier();
	requests->insert(requestIdentifier, request);
	return true;
}

/**
 * Remove a request from the manager without confirming it.
 *
 * @param identifier	The id of the request to cancel
 *
 * @return				true if the request existed, false otherwise
 */
bool RequestsManager::cancelRequest(QString identifier) {
	Request requestToDelete = requests->value(identifier);

	QSet<User> * usersNotInvited = requestToDelete.getUsers();

	QString messageRefused = MessageFormatter::cancellation(requestToDelete);

	QSet<User>::const_iterator i;
	for (i = usersNotInvited->begin(); i != usersNotInvited->end(); ++i){
		sender->sendUnicast(messageRefused, DeviceAddress((*i).getDeviceId()));
	}

	delete usersNotInvited;

	int deletionCount = requests->remove(identifier);
	return (deletionCount > 0);
}

/**
 * Confirms that the given request will occur and specify the activity chosen.
 *
 * @param identifier	The id of the confirmed request
 * @param activity		The activity that will happen
 *
 * @return 				true if the request existed, false otherwise.
 */
bool RequestsManager::confirmRequest(const QString& identifier, const Activity& activity, const QString& confirmationMsg) {
	if (requests->contains(identifier)) {
		Request requestToDelete = requests->value(identifier);

		QSet<User> * users = requestToDelete.getUsers(activity);
		QSet<User> * usersNotInvited = requestToDelete.getUsersNotIn(activity);

		QString messageInvited = MessageFormatter::confirmation(requestToDelete, activity, *users, confirmationMsg);
		QString messageRefused = MessageFormatter::cancellation(requestToDelete);

		QSet<User>::const_iterator i;
		for (i = users->begin(); i != users->end(); ++i){
			sender->sendUnicast(messageInvited, DeviceAddress((*i).getDeviceId()));
		}
		for (i = usersNotInvited->begin(); i != usersNotInvited->end(); ++i){
			sender->sendUnicast(messageRefused, DeviceAddress((*i).getDeviceId()));
		}
		QString date = QString::number(QDateTime::currentDateTime().toTime_t());
		ClientRequest request(requestToDelete, activity, date, confirmationMsg);
		ActivityLog::getInstance()->storeRequest(request);

		delete users;
		delete usersNotInvited;
	}

	int deletionCount = requests->remove(identifier);
	return (deletionCount > 0);
}


bool RequestsManager::removeRequest(const QString& identifier) {
	int deletionCount = requests->remove(identifier);
	return (deletionCount > 0);
}

/**
 * Returns the request associated with the given id
 *
 * @param identifier	The id of the wanted request
 *
 * @return 				The request associated
 */
Request * RequestsManager::getModifiableRequest(const QString& identifier) const {
	if (requests->contains(identifier)) {
		return &((*requests)[identifier]);
	} else {
		return new Request(Request::Invalid());
	}
}


Request RequestsManager::getConstRequest(const QString& identifier) const {
	if (requests->contains(identifier)) {
		return requests->value(identifier);
	} else {
		return Request::Invalid();
	}
}
